import { SQRT_OF_TWO, isZeroVector } from "./util.js"
import Actor from "./actor.js"

export default class Player extends Actor {
  constructor(game, x, y) {
    super(game)
    this.x = x
    this.y = y
    this.width = 32
    this.height = 64
    this.xVel = 0
    this.yVel = 0
    this.color = "#56E"
    this.playerDirection = { x: 0, y: 1 }
  }

  tick(dt) {
    this.handleInputAndMovement()
  }

  handleInputAndMovement() {
    const dir = this.inputDirection()
    this.xVel = dir.x * 2
    this.yVel = dir.y * 2

    this.x += this.xVel 
    this.y += this.yVel 

    if (!isZeroVector(dir)) this.playerDirection = dir
  }

  inputDirection() {
    const isInputPressed = this.game.input.isInputPressed.bind(this.game.input)
    const dir = { x: 0, y: 0 }
    if (isInputPressed("up")) dir.y -= 1
    if (isInputPressed("down")) dir.y += 1
    if (isInputPressed("left")) dir.x -= 1
    if (isInputPressed("right")) dir.x += 1

    if (Math.abs(dir.x, dir.y) == 2) {
      dir.x *= SQRT_OF_TWO
      dir.y *= SQRT_OF_TWO
    }

    return dir
  }

  draw(ctx) {
    this.color = `rgb(128 ${(this.playerDirection.x * 128) + 128} ${(this.playerDirection.y * 128) + 128}`
    ctx.beginPath()
    ctx.fillStyle = this.color
    ctx.rect(this.x, this.y, this.width, this.height)
    ctx.fill()
    ctx.closePath()
  }
}
